unit debugidle;
{
   TgfglDebugProcessPriority copyright Greg Lorriman Feb 2000
   http://www.lorriman.demon.co.uk greg@lorriman.demon.co.uk

    SHORT DESCRIPTION : sets app to idle priority to regain system 
   responsiveness in the case of high CPU usage apps, whilst generally 
   not losing noticeable app performance. Overcomes the inadequacies 
   of using application.processmessages along with a reliable mechanism 
   to avoid unintentional delivery/distribution of app in "idle" mode.

   LONG : CPU intensive apps slow down the rest of the system. With 
   this component you can avoid this slow down while having little 
   effect on the performance of the CPU intensive application. This 
   restores responsiveness to your system, making it usable while you 
   wait for the app to finish. This is done by setting the app's process 
   priority to "idle". This works because normally a system is idle 95%
   of the time. Since delivery/distribution of an app in "idle" mode is 
   probably highly undesirable the component has a mechanism for 
   avoiding this problem. The component has no methods or events.

   USAGE :

   The process priority is changed only when executed, not at design
   time.

   Generally properties IDLE and ENABLED must be set to true in order 
   for the app to have it's process priority set to idle on start-up. See 
   ALWAYSIDLE for the exception to this rule. Changing any of 
   ENABLED, IDLE and ALWAYSIDLE at runtime can change the 
   process priority even after startup.

   The delivery/distribution failsafe mechansim is supplied via the IDLE 
   property. It can be overriden (see below).

   ENABLED must be true for the process priority to be changeable.

   IDLE sets a key in the registry to "idle" or "normal" depending on
   whether it is true or false respectively. This is done whether or not the 
   component is ENABLED. If no key is found or it's value is not set to 
   "idle" then IDLE will be false. This property can be set at design time 
   in order to write the key without resorting to regedit. Changing IDLE 
   to true at runtime will override the delivery/distribution failsafe 
   mechansim.

   KEY is the registry path for the key used by the IDLE property. It's 
   rootkey is always HKEY_CURRENT_USER. Modify KEY to allow 
   "grouping" of your apps. Before installing the component you might 
   consider changing the default key path ("\software\myapps") by 
   editing the constant at the top of the component's unit (reinstall if 
   changing at a later date but note that this will not change the key in 
   apps for which the component is already in use).

   ALWAYSIDLE overrides the "idle" property (ie. even when there is 
   no registry key to confirm desired process priority) for those cases 
   when the developer expressly wants the app to be have idle process 
   priority on all systems. ENABLED must still be true to change 
   priority on application startup. Changing ALWAYSIDLE to true 
   overrides the delivery/distribution failsafe mechansim. Changing this
   property at design-time pops up a confirmation/warning box.

   SCROLL TO TOP TO SEE START OF HELP

   See readme.txt for disclaimer.
}

interface

uses classes;

const
   DEFAULT_KEY='\software\myapps';
   KEY_NAME='debugThreadMode';
   _VERSION='.2 beta';
   OTHER_STUFF='http://www.lorriman.demon.co.uk';

type
   TgfglDebugProcessPriority=class(TComponent)

   private

      FIdle: boolean;
      FEnabled: boolean;
      FKey: string;
      FHelp: TStrings;
      FOtherStuff: string;
      FVersion: string;
      FAlwaysIdle: boolean;
      procedure SetEnabled(const Value: boolean);
      procedure SetHelp(const Value: TStrings);
      procedure SetIdle(const Value: boolean);
      procedure SetKey(const Value: string);
      function getIdle: boolean;
      function getKey: string;
      procedure SetOtherStuff(const Value: string);
      procedure SetVersion(const Value: string);
      procedure SetAlwaysIdle(const Value: boolean);

   protected

      procedure loaded; override;
      procedure setPriority; virtual;

   public

      constructor create(AOwner: TComponent); override;
      destructor destroy; override;

   published

      property Enabled: boolean read FEnabled write SetEnabled;
      property Idle: boolean read getIdle write SetIdle stored false;
      property Key: string read getKey write SetKey;
      property AlwaysIdle: boolean read FAlwaysIdle write SetAlwaysIdle;

      property Help: TStrings read FHelp write SetHelp stored false;
      property Version: string read FVersion write SetVersion;
      property OtherStuff: string read FOtherStuff write SetOtherStuff;
   end;

procedure Register;

implementation

uses registry, windows, sysutils;

{ TgfglDebugProcessPriority }

constructor TgfglDebugProcessPriority.create(AOwner: TComponent);
const
   helpStr=
'TgfglDebugProcessPriority copyright Greg Lorriman Feb 2000 '#13#10+
'http://www.lorriman.demon.co.uk greg@lorriman.demon.co.uk'#13#10+
''#13#10+
'SHORT DESCRIPTION : sets app to idle priority to regain system '#13#10+
'responsiveness in the case of high CPU usage apps, whilst generally '#13#10+
'not losing noticeable app performance. Overcomes the inadequacies '#13#10+
'of using application.processmessages along with a reliable mechanism '#13#10+
'to avoid unintentional delivery/distribution of app in "idle" mode.'#13#10+
''#13#10+
'LONG : CPU intensive apps slow down the rest of the system. With '#13#10+
'this component you can avoid this slow down while having little '#13#10+
'effect on the performance of the CPU intensive application. This '#13#10+
'restores responsiveness to your system, making it usable while you '#13#10+
'wait for the app to finish. This is done by setting the app''s process '#13#10+
'priority to "idle". This works because normally a system is idle 95% '#13#10+
'of the time. Since delivery/distribution of an app in "idle" mode is '#13#10+
'probably highly undesirable the component has a mechanism for '#13#10+
'avoiding this problem. The component has no methods or events.'#13#10+
''#13#10+
'USAGE :'#13#10+
''#13#10+
'The process priority is changed only when executed, not at design '#13#10+
'time.'#13#10+
''#13#10+
'Generally properties IDLE and ENABLED must be set to true in order '#13#10+
'for the app to have it''s process priority set to idle on start-up. See '#13#10+
'ALWAYSIDLE for the exception to this rule. Changing any of '#13#10+
'ENABLED, IDLE and ALWAYSIDLE at runtime can change the '#13#10+
'process priority even after startup.'#13#10+
''#13#10+
'The delivery/distribution failsafe mechansim is supplied via the IDLE '#13#10+
'property. It can be overriden (see below).'#13#10+
''#13#10+
'ENABLED must be true for the process priority to be changeable.'#13#10+
''#13#10+
'IDLE sets a key in the registry to "idle" or "normal" depending on '#13#10+
'whether it is true or false respectively. This is done whether or not the '#13#10+
'component is ENABLED. If no key is found or it''s value is not set to '#13#10+
'"idle" then IDLE will be false. This property can be set at design time '#13#10+
'in order to write the key without resorting to regedit. Changing IDLE '#13#10+
'to true at runtime will override the delivery/distribution failsafe '#13#10+
'mechansim.'#13#10+
''#13#10+
'KEY is the registry path for the key used by the IDLE property. It''s '#13#10+
'rootkey is always HKEY_CURRENT_USER. Modify KEY to allow '#13#10+
'"grouping" of your apps. Before installing the component you might '#13#10+
'consider changing the default key path ("\software\myapps") by '#13#10+
'editing the constant at the top of the component''s unit (reinstall if '#13#10+
'changing at a later date but note that this will not change the key in '#13#10+
'apps for which the component is already in use).'#13#10+
''#13#10+
'ALWAYSIDLE overrides the "idle" property (ie. even when there is '#13#10+
'no registry key to confirm desired process priority) for those cases '#13#10+
'when the developer expressly wants the app to be have idle process '#13#10+
'priority on all systems. ENABLED must still be true to change '#13#10+
'priority on application startup. Changing ALWAYSIDLE to true '#13#10+
'overrides the delivery/distribution failsafe mechansim. Changing this '#13#10+
'property at design-time pops up a confirmation/warning box.'#13#10+
''#13#10+
'SCROLL TO TOP TO SEE START OF HELP'#13#10+
''#13#10+
'See readme.txt for disclaimer. ';

begin
   inherited create(aowner);
   fkey:=DEFAULT_KEY;
   Fenabled:=true;
   FHelp:=TStringlist.create;
   FHelp.text:=HelpStr;
   FVersion:=_VERSION;
   FOtherStuff:=OTHER_STUFF;
end;

destructor TgfglDebugProcessPriority.destroy;
begin
   fhelp.free;
   inherited;
end;

function TgfglDebugProcessPriority.getIdle: boolean;
var
   reg: TRegistry;
begin
   FIdle:=false;
   reg:=TRegistry.create;
   try
      reg.rootkey:=HKEY_CURRENT_USER;
      if reg.openkey(key, false)then begin
         if reg.valueExists(KEY_NAME)then
            FIdle:=lowercase(reg.readstring(KEY_NAME))='idle';
      end;
      reg.closeKey;
   finally
      reg.free;
   end;
   result:=fidle;
end;

function TgfglDebugProcessPriority.getKey: string;
begin
   result:=FKey;
end;

procedure TgfglDebugProcessPriority.loaded;
begin
   inherited loaded;
   setPriority;
end;

procedure TgfglDebugProcessPriority.SetAlwaysIdle(const Value: boolean);
const
   mess='Setting AlwaysIdle to true overrides the delivery/distribution fail-safe mechanism.'#13#10+
      'Are you certain you want to do this?';
begin
   if(value=true)and(csdesigning in componentState)and not(csLoading in componentstate)then begin
      if messagebox(0, mess, 'Overriding fail-safe mechanism', MB_YESNO+MB_ICONWARNING)<>IDYES then
         EXIT;
   end;
   FAlwaysIdle:=Value;
   setPriority;
end;

procedure TgfglDebugProcessPriority.SetEnabled(const Value: boolean);
begin
   if value=FEnabled then exit;
   FEnabled:=Value;

   if csLoading in componentstate then exit;

   setPriority;
end;

procedure TgfglDebugProcessPriority.SetHelp(const Value: TStrings);
begin

end;

procedure TgfglDebugProcessPriority.SetIdle(const Value: boolean);
var
   reg: TRegistry;
   str: string;
begin
   reg:=TRegistry.create;
   try
      reg.rootkey:=HKEY_CURRENT_USER;
      reg.openkey(key, true);
      if value then str:='idle' else str:='normal';
      reg.writestring(KEY_NAME, str);
      reg.closeKey;
   finally
      reg.free;
   end;
   setPriority;
end;

procedure TgfglDebugProcessPriority.SetKey(const Value: string);
var
   val: string;
begin
   val:=value;
   if val=fkey then exit;
   if val<>'' then
      if val[1]<>'\' then
         val:='\'+val;
   FKey:=Val;
end;

procedure TgfglDebugProcessPriority.SetOtherStuff(const Value: string);
begin

end;

procedure TgfglDebugProcessPriority.setPriority;
   function idleWanted: boolean;
   begin
      result:=enabled and(idle or alwaysIdle);
   end;
var
   p: integer;
begin
   if csDesigning in componentState then exit;
   if idleWanted then p:=IDLE_PRIORITY_CLASS
   else p:=NORMAL_PRIORITY_CLASS;
   setPriorityClass(GetCurrentProcess, p);
end;

procedure Register;
begin
   registercomponents('Custom', [TgfglDebugProcessPriority]);
end;

procedure TgfglDebugProcessPriority.SetVersion(const Value: string);
begin

end;

end.

